package nrs.jester;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.util.*;
import javax.swing.*;

import nrs.jester.db.CallFactory;

/*
 * Java Stored Procedure Tester (Jester)
 * Copyright (C) 2002 Nick Sydenham <nsydenham@yahoo.co.uk>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 */

/**
 * The ConnectionDialog is used by Jester to represent connections
 *
 * @author Nicholas Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class ConnectionDialog extends JDialog
{
    private Jester jester;
    private JTextField dbHostTF, dbPortTF, dbNameTF, userTF, spdefTF;
    private JPasswordField passwordTF;
    private JComboBox profileCB, dbTypeCB;
    private JCheckBox savePwdCB;
    private Map profiles;
    private ConnectionSetting cs = null;
    private Properties driverProps = null;
    
    /**
     * Create a connection dialog
     * @param jester owning app
     * @param cons ConnectionSetting
     */
    public ConnectionDialog(Jester jester, ConnectionSetting cons)
    {
	super(jester, "Connection Properties", true);
	
	this.jester = jester;
	profiles = jester.getDBConnection().getProfiles();
	
	Container content = getContentPane();
	GridBagLayout gridbag = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
	c.fill = GridBagConstraints.HORIZONTAL;
	content.setLayout(gridbag);
	
	c.gridwidth = 1;
	c.weightx = 0.0;
	c.insets = new Insets(3, 3, 3, 3);
	
	// first line
	content.add(jester.makeLabel("Profile name", gridbag, c));
	
	profileCB = new JComboBox();
	profileCB.setEditable(true);
	profileCB.addItem("<none>");
	profileCB.setToolTipText("Select an existing profile, or type "+
	    "in name of new profile");
	// add existing profiles
	Iterator it = profiles.keySet().iterator();
	String key;
	while (it.hasNext())
	{
	    key = (String)it.next();
	    profileCB.addItem(key);
	}
	profileCB.addItemListener(new ItemListener() {
	    public void itemStateChanged(ItemEvent e) {
		selectProfile();
	    }
	});
	gridbag.setConstraints(profileCB, c);
	content.add(profileCB);
	
	JButton newProfile = new JButton("New");
	newProfile.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		newProfile();
	    }
	});
	c.gridwidth = GridBagConstraints.REMAINDER;
	gridbag.setConstraints(newProfile, c);
	content.add(newProfile);
	
	// second line
	c.gridwidth = 1;
	content.add(jester.makeLabel("Database type", gridbag, c));
	dbTypeCB = new JComboBox(CallFactory.getConnectionTypes());
	gridbag.setConstraints(dbTypeCB, c);
	content.add(dbTypeCB);
	
	JButton delProfile = new JButton("Delete");
	delProfile.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		String pName = (String)profileCB.getSelectedItem();
		if (!pName.equals("<none>")) {
		    driverProps = null;
		    profiles.remove(pName);
		    profileCB.removeItem(pName);
		} else {
		    Toolkit.getDefaultToolkit().beep();
		}
	    }
	});
	c.gridwidth = GridBagConstraints.REMAINDER;
	gridbag.setConstraints(delProfile, c);
	content.add(delProfile);
	
	// line 3
	c.gridwidth = 1;
	content.add(jester.makeLabel("Hostname", gridbag, c));
	dbHostTF = new JTextField();
	c.gridwidth = GridBagConstraints.REMAINDER;
	gridbag.setConstraints(dbHostTF, c);
	content.add(dbHostTF);
	
	// line 4
	c.gridwidth = 1;
	content.add(jester.makeLabel("Port", gridbag, c));
	dbPortTF = new JTextField();
	dbPortTF.setToolTipText("Leave blank to use the default port");
	c.gridwidth = GridBagConstraints.REMAINDER;
	gridbag.setConstraints(dbPortTF, c);
	content.add(dbPortTF);
	
	// line 5
	c.gridwidth = 1;
	content.add(jester.makeLabel("Database name", gridbag, c));
	dbNameTF = new JTextField();
	c.gridwidth = GridBagConstraints.REMAINDER;
	gridbag.setConstraints(dbNameTF, c);
	content.add(dbNameTF);
	
	// line 6
	c.gridwidth = 1;
	content.add(jester.makeLabel("User", gridbag, c));
	userTF = new JTextField();
	c.gridwidth = GridBagConstraints.REMAINDER;
	gridbag.setConstraints(userTF, c);
	content.add(userTF);
	
	// line 7
	c.gridwidth = 1;
	content.add(jester.makeLabel("Password", gridbag, c));
	passwordTF = new JPasswordField();
	gridbag.setConstraints(passwordTF, c);
	content.add(passwordTF);
	
	savePwdCB = new JCheckBox("Save pwd");
	savePwdCB.setToolTipText("Select to save the password");
	c.gridwidth = GridBagConstraints.REMAINDER;
	gridbag.setConstraints(savePwdCB, c);
	content.add(savePwdCB);
	
	// line 8
	c.gridwidth = 1;
	content.add(jester.makeLabel("SP Definition", gridbag, c));
	spdefTF = new JTextField();
	spdefTF.setEditable(false);
	spdefTF.setToolTipText("Optional: stored procedure definition file");
	gridbag.setConstraints(spdefTF, c);
	content.add(spdefTF);
	
	JButton spDefButton = new JButton("Select");
	spDefButton.setToolTipText("Click to select stored procedure definition file");
	spDefButton.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		selectSPDefFile();
	    }
	});
	c.gridwidth = GridBagConstraints.REMAINDER;
	gridbag.setConstraints(spDefButton, c);
	content.add(spDefButton);
	
	// button panel - line 9
	c.gridwidth = 3;
	c.weighty = 1.0;
	c.anchor = GridBagConstraints.SOUTH;
	
	JPanel buttonPanel = new JPanel();
	JButton connectButton = new JButton("Connect");
	connectButton.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		if (setConnection()) setVisible(false);
	    }
	});
	buttonPanel.add(connectButton);

	JButton propsButton = new JButton("Properties...");
	propsButton.setToolTipText("Set database specific connection properties");
	propsButton.addActionListener(new ActionListener() {
	   public void actionPerformed(ActionEvent e) {
	       getDriverProperties();
	   }
	});
	buttonPanel.add(propsButton);
	
	JButton cancelButton = new JButton("Cancel");
	cancelButton.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		setVisible(false);
		cs = null;
	    }
	});
	buttonPanel.add(cancelButton);

	gridbag.setConstraints(buttonPanel, c);
	content.add(buttonPanel);

	// set values to existing setting or first profile if available
	if (cons != null)
	{
	    profileCB.setSelectedItem(cons.profileName);
	    dbHostTF.setText(cons.host);
	    dbNameTF.setText(cons.dbName);
	    dbTypeCB.setSelectedIndex(cons.dbType);
	    userTF.setText(cons.user);
	    passwordTF.setText(cons.password);
	    savePwdCB.setSelected(cons.savePassword);
	    spdefTF.setText(cons.spDefFile);
	}
	else if (profiles.size() > 0)
	{
	    it = profiles.keySet().iterator();
	    String pName = (String)it.next();
	    cs = (ConnectionSetting)profiles.get(pName);
	    profileCB.setSelectedItem(pName);
	    dbHostTF.setText(cs.host);
	    dbNameTF.setText(cs.dbName);
	    dbTypeCB.setSelectedIndex(cs.dbType);
	    userTF.setText(cs.user);
	    passwordTF.setText(cs.password);
	    savePwdCB.setSelected(cs.savePassword);
	    spdefTF.setText(cs.spDefFile);
	}
	
	pack();
	setLocationRelativeTo(jester);
	setVisible(true);
	
	connectButton.requestFocus();
    }
    
    /**
     * Get the selected ConnectionSetting
     * @return ConnectionSetting
     */
    public ConnectionSetting getConnectionSetting()
    {
	return cs;
    }
    
    /**
     * Set the connection settings
     */
    private boolean setConnection()
    {
	String pName = (String)profileCB.getSelectedItem();
	if (pName.length() == 0 || pName.equalsIgnoreCase("<none>"))
	{
	    JOptionPane.showMessageDialog(this, "Please enter a profile name", "No profile", JOptionPane.ERROR_MESSAGE);
	    return false;
	}
	
	if (dbNameTF.getText().length() == 0)
	{
	    JOptionPane.showMessageDialog(this, "Please enter a database name", "No database", JOptionPane.ERROR_MESSAGE);
	    return false;
	}
	
	if (dbHostTF.getText().length() == 0)
	{
	    JOptionPane.showMessageDialog(this, "Please enter a host", "No host", JOptionPane.ERROR_MESSAGE);
	    return false;
	}
	
	cs = new ConnectionSetting();
	cs.profileName = pName;
	cs.dbType = dbTypeCB.getSelectedIndex();
	cs.dbName = dbNameTF.getText();
	cs.host = dbHostTF.getText();
	cs.driverProperties = driverProps;
	
	String tempPort = dbPortTF.getText();
	if (tempPort.length() > 0)
	{
	    try {
		cs.port = Integer.parseInt(tempPort);
	    } catch (NumberFormatException nfex) {
		JOptionPane.showMessageDialog(this, "Invalid port number "+tempPort, "Invalid port", JOptionPane.ERROR_MESSAGE);
		return false;
	    }
	}
	
	if (userTF.getText().length() > 0) cs.user = userTF.getText();
	if (passwordTF.getPassword().length > 0) {
	    cs.password = new String(passwordTF.getPassword());
	}
	cs.savePassword = savePwdCB.isSelected();
	if (spdefTF.getText().length() > 0) cs.spDefFile = spdefTF.getText();
	
	profiles.put(cs.profileName, cs);
	
	jester.setStatus("Connecting...");
	
	return true;
    }

    /** User selected a profile from the list */
    private void selectProfile()
    {
	String pName = (String)profileCB.getSelectedItem();
	if (pName.equalsIgnoreCase("<none>"))
	{
	    dbHostTF.setText(null);
	    dbPortTF.setText(null);
	    dbNameTF.setText(null);
	    dbTypeCB.setSelectedIndex(0);
	    userTF.setText(null);
	    passwordTF.setText(null);
	    savePwdCB.setSelected(false);
	    spdefTF.setText(null);
	    driverProps = null;
	} 
	else
	{
	    cs = (ConnectionSetting)profiles.get(pName);
	    if (cs != null)
	    {
		dbHostTF.setText(cs.host);
		if (cs.port > 0) dbPortTF.setText(Integer.toString(cs.port));
		dbNameTF.setText(cs.dbName);
		dbTypeCB.setSelectedIndex(cs.dbType);
		userTF.setText(cs.user);
		passwordTF.setText(cs.password);
		savePwdCB.setSelected(cs.savePassword);
		spdefTF.setText(cs.spDefFile);
		driverProps = cs.driverProperties;
	    }
	}
    }

    /** User wants to enter a new profile */
    private void newProfile()
    {
	dbHostTF.setText(null);
	dbPortTF.setText(null);
	dbNameTF.setText(null);
	userTF.setText(null);
	passwordTF.setText(null);
	savePwdCB.setSelected(false);
	profileCB.setSelectedIndex(0);
	driverProps = null;
    }
    
    /** Allow the user to enter driver specific properties */
    private void getDriverProperties()
    {
	if (cs != null) driverProps = cs.driverProperties;
	String[] connProps = CallFactory.getConnectionProperties(dbTypeCB.getSelectedIndex());
	DriverPropertiesDialog dpd = new DriverPropertiesDialog(this, driverProps, connProps);
	if (dpd.getReturnStatus() == DriverPropertiesDialog.OK) {
	    driverProps = dpd.getDriverProperties();
	}
	dpd.dispose();
    }
    
    /** Select a stored procedure definition file */
    private void selectSPDefFile()
    {
	JFileChooser fc = new JFileChooser();
	int resp = fc.showOpenDialog(this);
	if (resp == JFileChooser.APPROVE_OPTION) {
	    File spFile = fc.getSelectedFile();
	    spdefTF.setText(spFile.getAbsolutePath());
	} else {
	    spdefTF.setText(null);
	}
    }
}
