package nrs.jester;

import java.util.*;

import nrs.jester.db.CallFactory;

/*
 * Java Stored Procedure Tester (Jester)
 * Copyright (C) 2002 Nick Sydenham <nsydenham@yahoo.co.uk>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 */

/**
 * A ConnectionSetting represents a connection to a DBMS
 *
 * @author Nicholas Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class ConnectionSetting implements Comparable
{
    public String profileName = null;
    public int dbType = CallFactory.AS400;
    public String host = null;
    public int port = 0;
    public String dbName = null;
    public String user = null;
    public String password = null;
    public boolean savePassword = false;
    public String spDefFile = null;
    public Properties driverProperties = null;
    
    /**
     * Compares this object with the specified object for order.  Returns a
     * negative integer, zero, or a positive integer as this object is less
     * than, equal to, or greater than the specified object.<p>
     *
     * The implementor must ensure <tt>sgn(x.compareTo(y)) ==
     * -sgn(y.compareTo(x))</tt> for all <tt>x</tt> and <tt>y</tt>.  (This
     * implies that <tt>x.compareTo(y)</tt> must throw an exception iff
     * <tt>y.compareTo(x)</tt> throws an exception.)<p>
     *
     * The implementor must also ensure that the relation is transitive:
     * <tt>(x.compareTo(y)&gt;0 &amp;&amp; y.compareTo(z)&gt;0)</tt> implies
     * <tt>x.compareTo(z)&gt;0</tt>.<p>
     *
     * Finally, the implementer must ensure that <tt>x.compareTo(y)==0</tt>
     * implies that <tt>sgn(x.compareTo(z)) == sgn(y.compareTo(z))</tt>, for
     * all <tt>z</tt>.<p>
     *
     * It is strongly recommended, but <i>not</i> strictly required that
     * <tt>(x.compareTo(y)==0) == (x.equals(y))</tt>.  Generally speaking, any
     * class that implements the <tt>Comparable</tt> interface and violates
     * this condition should clearly indicate this fact.  The recommended
     * language is "Note: this class has a natural ordering that is
     * inconsistent with equals."
     *
     * @param   o the Object to be compared.
     * @return  a negative integer, zero, or a positive integer as this object
     *		is less than, equal to, or greater than the specified object.
     *
     * @throws ClassCastException if the specified object's type prevents it
     *         from being compared to this Object.
     */
    public int compareTo(Object o)
    {
	ConnectionSetting cs2 = (ConnectionSetting)o;
	
	if (profileName != null)
	{
	    if (cs2.profileName != null)
	    {
		return profileName.compareTo(cs2.profileName);
	    } else {
		return 1;
	    }
	}
	
	return 0;
    }
    
    /**
     * Get a string representation of this connection which
     * is written to the connections file.
     */
    public String toString()
    {
	StringBuffer sb = new StringBuffer("dbType=");
	sb.append(dbType);
	sb.append("\n");
	
	sb.append("host=");
	sb.append(host);
	sb.append("\n");
	
	sb.append("port=");
	sb.append(port);
	sb.append("\n");
	
	sb.append("dbName=");
	sb.append(dbName);
	sb.append("\n");
	
	if (user != null) {
	    sb.append("user=");
	    sb.append(user);
	    sb.append("\n");
	}
	
	if (savePassword && password != null)
	{
	    sb.append("password=");
	    sb.append(password);
	    sb.append("\n");
	}
	
	if (spDefFile != null) {
	    sb.append("spDefFile=");
	    sb.append(spDefFile);
	    sb.append("\n");
	}
	
	if (driverProperties != null) {
	    String key;
	    Iterator it = driverProperties.keySet().iterator();
	    while (it.hasNext()) {
		key = (String)it.next();
		sb.append(key);
		sb.append("=");
		sb.append(driverProperties.get(key));
		sb.append("\n");
	    }
	}
	
	return sb.toString();
    }
}
