package nrs.jester;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;

/*
 * Java Stored Procedure Tester (Jester)
 * Copyright (C) 2002 Nick Sydenham <nsydenham@yahoo.co.uk>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 */

/**
 * Panel for editing parameter names and SP description
 *
 * @author Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class ParameterNamesPanel extends JPanel
{
    private Jester jester = null;
    private StoredProcedure sp = null;
    private JTextField[] inputs;
    private JTextField[] outputs;
    private JTextField[] inTips;
    private JTextField spDescription = null;
    
    private final static int IN = 0;
    private final static int OUT = 1;
    
    /** Creates a new instance of ParameterNamesPanel */
    public ParameterNamesPanel(Jester jester, StoredProcedure sp)
    {
	Parameter param;
	Iterator it;
	java.util.List paramList;
	int i;
	
	this.jester = jester;
	this.sp = sp;
	
	GridBagLayout gbl = new GridBagLayout();
	GridBagConstraints gbc = new GridBagConstraints();
	setLayout(gbl);
	
	gbc.gridwidth = 1;
	gbc.weightx = 0.0;
	gbc.insets = new Insets(5, 5, 5, 5);
	add(jester.makeLabel("Description", new Font("SansSerif", Font.PLAIN, 12), gbl, gbc));
	
	spDescription = new JTextField();
	spDescription.setText(sp.getDescription());
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	gbc.weightx = 1.0;
	gbc.fill = GridBagConstraints.HORIZONTAL;
	gbl.setConstraints(spDescription, gbc);
	add(spDescription);
	
	// add input parameters
	if (sp.getInParameters().size() > 0)
	{
	    paramList = sp.getInParameters();
	    inputs = new JTextField[paramList.size()];
	    inTips = new JTextField[paramList.size()];
	    createInputs(IN, paramList, gbl, gbc);
	}
	
	if (sp.getOutParameters().size() > 0)
	{
	    paramList = sp.getOutParameters();
	    outputs = new JTextField[paramList.size()];
	    createInputs(OUT, paramList, gbl, gbc);
	}
	
	// create edit buttons
	JPanel buttonPanel = new JPanel();
	buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
	
	JButton cancelButton = new JButton("Cancel");
	cancelButton.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		cancel();
	    }
	});
	buttonPanel.add(cancelButton);
	buttonPanel.add(Box.createHorizontalGlue());
	
	JButton updateButton = new JButton("Update");
	updateButton.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		updateSP();
	    }
	});
	buttonPanel.add(updateButton);
	
	gbc.weighty = 1.0;
	gbc.fill = GridBagConstraints.BOTH;
	gbc.anchor = GridBagConstraints.SOUTH;
	gbl.setConstraints(buttonPanel, gbc);
	add(buttonPanel);
    }
    
    /**
     * Create the inputs
     * @param type either IN or OUT
     * @param params list of parameters to create
     * @param gbl GridBagLayout
     * @param gbc GridBagConstraints
     */
    private void createInputs(int type, java.util.List params, GridBagLayout gbl, GridBagConstraints gbc)
    {
	JTextField paramDesc, tip=null;
	JLabel lengthLabel;
	Parameter param;
	Font titleFont = new Font("SansSerif", Font.BOLD, 14);
	Font labelFont = new Font("SansSerif", Font.PLAIN, 12);
	String label = (type == IN ? "In parameters" : "Out parameters");
	
	this.add(jester.makeLabel(label, titleFont, gbl, gbc));
	    
	gbc.gridwidth = 1;
	gbc.weightx = 0.0;
	gbc.anchor = GridBagConstraints.WEST;
	gbc.fill = GridBagConstraints.NONE;
	this.add(jester.makeLabel("", gbl, gbc));

	gbc.weightx = 1.0;
	gbc.fill = GridBagConstraints.HORIZONTAL;
	this.add(jester.makeLabel("Parameter description", labelFont, gbl, gbc));

	if (type == IN) {
	    this.add(jester.makeLabel("Tip", labelFont, gbl, gbc));
	}
	
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	this.add(jester.makeLabel("Length", labelFont, gbl, gbc));

	Iterator it = params.iterator();
	int i = 1;
	while (it.hasNext())
	{
	    gbc.gridwidth = 1;
	    gbc.weightx = 0.0;
	    gbc.fill = GridBagConstraints.NONE;
	    this.add(jester.makeLabel(Integer.toString(i), labelFont, gbl, gbc));

	    param = (Parameter)it.next();
	    paramDesc = new JTextField(param.getDescription(), 10);
	    gbc.weightx = 1.0;
	    gbc.fill = GridBagConstraints.HORIZONTAL;
	    gbl.setConstraints(paramDesc, gbc);
	    this.add(paramDesc);

	    if (type == IN) {
		tip = new JTextField(param.getTip(), 10);
		gbl.setConstraints(tip, gbc);
		this.add(tip);
	    }

	    gbc.gridwidth = GridBagConstraints.REMAINDER;
	    lengthLabel = jester.makeLabel(Integer.toString(param.getLength()), labelFont, gbl, gbc);
	    lengthLabel.setAlignmentX(SwingConstants.CENTER);
	    this.add(lengthLabel);
	    
	    switch (type)
	    {
		case IN:
		    inputs[i-1] = paramDesc;
		    inTips[i-1] = tip;
		    break;
		    
		case OUT:
		    outputs[i-1] = paramDesc;
		    break;
	    }
	    
	    i++;
	}
    }
    
    /**
     * Update the Stored Procedure definition
     */
    private void updateSP()
    {
	int i;
	Iterator it;
	Parameter param;
	String desc;
	
	sp.setDescription(spDescription.getText());
	
	java.util.List inParams = sp.getInParameters();
	it = inParams.iterator();
	i = 0;
	while (it.hasNext()) {
	    param = (Parameter)it.next();
	    desc = inputs[i].getText();
	    if (desc.length() == 0) {
		jester.setStatus("Input parameter "+(i+1)+" descripton cannot be blank");
		return;
	    }
	    param.setDescription(desc);
	    param.setTip(inTips[i].getText());
	    i++;
	}
	
	java.util.List outParams = sp.getOutParameters();
	it = outParams.iterator();
	i = 0;
	while (it.hasNext()) {
	    param = (Parameter)it.next();
	    param.setDescription(outputs[i].getText());
	    i++;
	}
	
	int resp = JOptionPane.showConfirmDialog(this, "Save changes to file?",
	    "Save changes?", JOptionPane.YES_NO_OPTION);
	if (resp == JOptionPane.YES_OPTION) {
	    if (!jester.saveSPDefs(false)) return;
	}
	
	jester.setStatus("");
	jester.setupInputs();
    }
    
    /** Cancel the edit */
    private void cancel()
    {
	jester.setupInputs();
    }
}
