package nrs.jester;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;

public class QueryPanel extends JPanel
{
    private Jester jester;
    private JTextArea queryArea;
    private JComboBox queryList;
    private File sqlFile = null;
    private java.util.List previousQueries = new ArrayList();
    private final static int MAX_NUM_QUERIES = 15;
    
    public QueryPanel(Jester jester)
    {
	this.jester = jester;
	
        setLayout(new BorderLayout());
        
	// query history
	JPanel histPanel = new JPanel();
	histPanel.setLayout(new BoxLayout(histPanel, BoxLayout.X_AXIS));
	JLabel histLabel = new JLabel("Query history:");
	histLabel.setBorder(BorderFactory.createEmptyBorder(3, 5, 3, 5));
	histPanel.add(histLabel);
	
	queryList = new JComboBox();
	queryList.setEditable(false);
	queryList.setBorder(BorderFactory.createEmptyBorder(3, 0, 3, 5));
	queryList.setMaximumSize(new Dimension(500, 30));
	queryList.addItemListener(new ItemListener() {
	    public void itemStateChanged(ItemEvent e) {
		int qIndex = queryList.getSelectedIndex();
		if (qIndex > -1 && previousQueries.size() > 0)
		    queryArea.setText((String)previousQueries.get(qIndex));
	    }
	});
	histPanel.add(queryList);

	add(histPanel, BorderLayout.NORTH);
	
	// query area
        queryArea = new JTextArea(5, 50);
        queryArea.setCursor(new Cursor(Cursor.TEXT_CURSOR));
        JScrollPane scrollQuery = new JScrollPane(queryArea);
        add(scrollQuery, BorderLayout.CENTER);
        
	// buttons
	JPanel buttonPanel = new JPanel();
	
	JButton load = new JButton("Load SQL", new ImageIcon(jester.getImage(Jester.OPEN_ICON)));
	load.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		loadSQL();
	    }
	});
	buttonPanel.add(load);
	
	JButton save = new JButton("Save SQL", new ImageIcon(jester.getImage(Jester.SAVE_ICON)));
	save.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		saveSQL();
	    }
	});
	buttonPanel.add(save);
	
        JButton action = new JButton("Execute", new ImageIcon(jester.getImage(Jester.GO_ICON)));
        action.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
		executeQuery();
            }
	});
	buttonPanel.add(action);
	
	add(buttonPanel, BorderLayout.SOUTH);
    }
 
    /** Execute the query */
    private void executeQuery()
    {
	jester.setStatus("Performing query");
	
	String sql = queryArea.getText();
	jester.performQuery(queryArea.getText());
	String key;
	if (sql.length() > 40) {
	    key = sql.substring(0, 40);
	} else {
	    key = sql;
	}
	
	// limit length of list
	if (previousQueries.size() >= MAX_NUM_QUERIES)
	{
	    queryList.remove(0);
	    previousQueries.remove(0);
	}
	
	queryList.addItem(key);
	previousQueries.add(sql);
    }
    
    /**
     * Add some text at the caret position
     * @param text the text to add
     */
    public void addToQuery(String text)
    {
	queryArea.insert(text, queryArea.getCaretPosition());
    }
    
    /** Load an SQL script */
    private void loadSQL()
    {
	JFileChooser open; 
	if (sqlFile != null) open = new JFileChooser(sqlFile);
	else open = new JFileChooser();
	
	int resp = open.showOpenDialog(jester);
	if (resp == JFileChooser.APPROVE_OPTION) {
	    try {
		BufferedReader reader = new BufferedReader(new FileReader(open.getSelectedFile()));
		String line;
		StringBuffer sb = new StringBuffer();
		while ((line = reader.readLine()) != null) {
		    sb.append(line);
		    sb.append("\n");
		}
		reader.close();
		queryArea.setText(sb.toString());
	    } catch (IOException ioex) {
		jester.setStatus("Cannot load SQL file: "+ioex.getMessage());
	    }
	}
    }
    
    /** Save an SQL script */
    private void saveSQL()
    {
	JFileChooser save = new JFileChooser();
	if (sqlFile != null) save.setSelectedFile(sqlFile);
	int resp = save.showSaveDialog(jester);
	if (resp == JFileChooser.APPROVE_OPTION) {
	    try {
		BufferedWriter writer = new BufferedWriter(new FileWriter(save.getSelectedFile()));
		writer.write(queryArea.getText());
		writer.close();
		jester.setStatus("Saved SQL to "+save.getSelectedFile().getAbsolutePath());
	    } catch (IOException ioex) {
		jester.setStatus("Couldn't save SQL to file: "+ioex.getMessage());
	    }
	}
    }
}