package nrs.jester.db;

import java.sql.*;
import java.util.Properties;

import nrs.jester.ConnectionSetting;

/*
 * Java Stored Procedure Tester (Jester)
 * Copyright (C) 2002 Nick Sydenham <nsydenham@yahoo.co.uk>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 */

/**
 * DB2 Call implementation
 *
 * @author Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class DB2Call extends DBCall
{
    private String DRIVER = "COM.ibm.db2.jdbc.app.DB2Driver";
    private String URL = "jdbc:db2";
    // IBM documentation is impenetrable
    private final static String[] connProps = new String[0];
    
    /** Get a Connection to the database
     * @param cs ConnectionSetting
     * @param timeout login timeout
     * @return an SQL connection
     * @exception java.sql.SQLException if a connection can't be made
     */
    public Connection getConnection(ConnectionSetting cs, int timeout) throws SQLException
    {
	try {
            Class.forName(DRIVER);
        } catch (ClassNotFoundException e) {
            // A call to Class.forName() forces us to consider this exception :-)...
            throw new SQLException("DB2 JDBC driver not in classpath");
        }
	
	StringBuffer sb = new StringBuffer(URL);
	sb.append(":");
	sb.append(cs.host);
	if (cs.port > 0) {
	    sb.append(":");
	    sb.append(cs.port);
	}
	sb.append(":");
	sb.append(cs.dbName);
	
        return makeConnection(sb.toString(), cs, timeout);
    }
    
    /** Get a ResultSet from the call
     * @return java.sql.ResultSet
     */
    public ResultSet getResultSet() throws SQLException
    {
	return call.getResultSet();
    }
    
    /** Prepare a stored procedure call
     * @param conn the connection to the database
     * @param procedure the stored procedure name including whatever package it's in
     * @param inputParams the number of input parameters
     * @param outputTypes the output types
     * @exception SQLException if it goes wrong
     */
    public void prepareCall(Connection conn, String procedure, int inputParams, int[] outputTypes) throws SQLException
    {
	int i;
        
        StringBuffer sql = new StringBuffer("call ");
        sql.append(procedure);
        sql.append(" (");
        
        for (i=0; i<inputParams; i++) {
            sql.append("?,");
        }
        
        for (i=0; i<outputTypes.length; i++) {
            sql.append("?,");
        }
        
        if (inputParams>0 || outputTypes.length>0)
            sql.setLength(sql.length()-1); // remove the last comma
        
        sql.append(")");
        
        call = conn.prepareCall(sql.toString());
        
        // register the output parameters
        for (i=0; i<outputTypes.length; i++) {
            call.registerOutParameter(inputParams+i+1, outputTypes[i]);
        }
    }
    
    /**
     * Get optional connection properties
     * @return an array of connection property names
     */
    public String[] getConnectionProperties() {
	return connProps;
    }
}
