package nrs.jester.db;

import java.sql.*;
import java.util.Properties;

import nrs.jester.ConnectionSetting;

/*
 * Java Stored Procedure Tester (Jester)
 * Copyright (C) 2002 Nick Sydenham <nsydenham@yahoo.co.uk>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 */

/**
 * Sybase base implementation of DBCall
 *
 * @author Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public abstract class SybaseBaseCall extends DBCall
{
    private final static String URL = "jdbc:sybase:Tds";
    protected String _DRIVER = null;
    private String[] connProps = {
	"applicationname", "autogen_cursor_name", "cancel_all", "charset",
	"charset_converter", "charset_converter_class", "dynamic_prepare",
	"expirestring", "hostproc", "jconnect_version", "language",
	"language_cursor", "literal_params", "packetsize", "protocol_capture",
	"proxy", "remotepwd", "repeat_read", "servicename", "session_id",
	"session_timeout", "sqlinitstring", "stream_cache_size",
    };
    
    /** Get a Connection to the database
     * @param cs ConnectionSetting
     * @param timeout login timeout
     * @return an SQL connection
     * @exception java.sql.SQLException if a connection can't be made
     */
    public Connection getConnection(ConnectionSetting cs, int timeout) throws SQLException
    {
        try {
            Class.forName(_DRIVER);
        } catch (ClassNotFoundException e) {
            // A call to Class.forName() forces us to consider this exception :-)...
            throw new SQLException("Sybase JDBC driver not in classpath");
        }

	StringBuffer sb = new StringBuffer(URL);
	sb.append(":");
	sb.append(cs.host);
	if (cs.port > 0) {
	    sb.append(":");
	    sb.append(cs.port);
	}
	sb.append("/");
	sb.append(cs.dbName);
	
	return makeConnection(sb.toString(), cs, timeout);
    }
    
    /** Get a ResultSet from the call
     * @return java.sql.ResultSet
     */
    public ResultSet getResultSet() throws SQLException
    {
	return call.getResultSet();
    }
    
    /** Prepare a stored procedure call
     * @param conn the connection to the database
     * @param procedure the stored procedure name including whatever package it's in
     * @param inputParams the number of input parameters
     * @param outputTypes the output types
     * @exception SQLException if it goes wrong
     */
    public void prepareCall(Connection conn, String procedure, int inputParams, int[] outputTypes) throws SQLException
    {
	int i;
        
        StringBuffer sql = new StringBuffer("{call ");
        sql.append(procedure);
        
        if( inputParams > 0 ) {
            sql.append(" (");
            
            for (i=0; i<inputParams; i++) {
                sql.append("?,");
            }
            
            if (inputParams>0)
                sql.setLength(sql.length()-1); // remove the last comma
            
            sql.append(")");
        }
        
        sql.append("}");
        
        call = conn.prepareCall(sql.toString());
    }
    
    public String[] getConnectionProperties()
    {
	return connProps;
    }
}
